/************************************************************************
 *   IRC - Internet Relay Chat, modules/m_sgline.c
 *
 *   Copyright (C) 2000-2003 TR-IRCD Development
 *
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * $Id: m_sgline.c,v 1.4 2003/06/14 13:55:51 tr-ircd Exp $ 
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "s_conf.h"

static char *token = TOK1_SGLINE;

static struct Message _msgtab[] = {
    {MSG_SGLINE, 0, MAXPARA, M_SLOW, 0L,
     m_unregistered, m_ignore, m_ignore, m_sgline, m_sgline}
};

#ifndef STATIC_MODULES

char *_version = "$Revision: 1.4 $";

void _modinit(void)
{
    mod_add_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = _msgtab;
}

void _moddeinit(void)
{
    mod_del_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = NULL;
}
#else
void m_sgline_init(void)
{
    mod_add_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = _msgtab;
}
#endif

int m_sgline(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    aMaskItem *ami;
    int len;
    char *mask, *reason;

    if (!(IsServer(sptr) || IsULine(sptr)))
	return 0;

    if (parc < 3) 
	return 0;

    len = atoi(parv[1]);
    mask = parv[2];
    if ((strlen(mask) > len) && (mask[len]) == ':') {
	mask[len] = '\0';
	reason = mask + len + 1;
    } else {			/* Bogus */
	return 0;
    }

    if ((ami = find_maskitem(parv[1], NULL, MASKITEM_GECOS, 1)) == NULL) {

	ami = create_maskitem(sptr->name, mask, NULL, MASKITEM_GECOS, 0);
        if (!ami)
            return -1;

	DupString(ami->reason, reason ? reason : "Reason Unspecified");

    }
    sendto_serv_butone(cptr, sptr, TOK1_SGLINE, "%d :%s:%s", len, mask, reason);
    return 0;
}

