/************************************************************************
 *   IRC - Internet Relay Chat, modules/operdo/m_operdo.c
 *
 *   Copyright (C) 2000-2003 TR-IRCD Development
 *
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "chanmode.h"
#include "s_conf.h"
#include "h.h"
#include "hook.h"

static int hookid_operdo_core = 0;

static struct Message _msgtab[] = {
    {MSG_OPERDO, 0, MAXPARA, M_SLOW, 0L,
     m_unregistered, m_permission, m_operdo, m_ignore, m_ignore}
};

#ifndef STATIC_MODULES

char *_version = "$Revision: 1.3 $";

void _modinit(void)
{
    hookid_operdo_core = hook_add_event("load operdo core module");
    mod_add_cmd(_msgtab);
}

void _moddeinit(void)
{
    hook_del_event("load operdo core module");
    mod_del_cmd(_msgtab);
}
#else
void m_core_init(void)
{
    hookid_operdo_core = hook_add_event("load operdo core module");
    mod_add_cmd(_msgtab);
}
#endif

int m_operdo(aClient *cptr, aClient *sptr, int parc, char *parv[])
{

    aChannel *chptr = NULL;
    char *o_param;
    struct hook_data thisdata;

    if (!OPIsOperdo(sptr))
	return m_permission(cptr, sptr, parc, parv);

    o_param = parv[1];

    if (o_param == NULL)
	o_param = "HELP";

    thisdata.client_p = cptr;
    thisdata.source_p = sptr;
    thisdata.parc = parc;
    thisdata.parv = parv;

    if (!irc_strcmp(o_param, "HELP")) {
	thisdata.extra = o_param;
	if (!hook_call_event(hookid_operdo_core, &thisdata)) {
	    send_me_notice(sptr, "No such OPERDO command %s", parv[1]);
	    send_me_notice(sptr, ":Try \002HELP\002");
	}
	return 0;
    }

    if (parc < 3) {
	send_me_numeric(sptr, ERR_NEEDMOREPARAMS, MSG_OPERDO);
	return 0;
    }

    if (!check_channelname(sptr, (unsigned char *) parv[2])) {
	send_me_numeric(sptr, ERR_NOSUCHCHANNEL, parv[2]);
	return 0;
    }
    chptr = find_channel(parv[2]);
    if (!chptr) {
	send_me_numeric(sptr, ERR_NOSUCHCHANNEL, parv[2]);
	return 0;
    }

    thisdata.channel = chptr;

    if (IsChanAnon(chptr)) {
	send_me_notice(sptr, ":OPERDO Cannot be used for anonymous channels");
	sendto_ops("from %C: %C tried to use OPERDO on anonymous channel %H", &me, sptr, chptr);
	return 0;
    }

    thisdata.extra = o_param;
    if (!hook_call_event(hookid_operdo_core, &thisdata)) {
	logevent_call(LogSys.operevent, MSG_OPERDO, sptr, &parv, parc);
	send_me_notice(sptr, "No such OPERDO command %s", parv[1]);
	send_me_notice(sptr, ":Try \002HELP\002");
    }
    return 0;

}
